package teslainventory_sdk

const (
	// Version is the current version of the sdk.
	Version              = "1.0.0"
	inventoryApiEndpoint = "https://www.tesla.com/inventory/api/"
	defaultUserAgent     = "TeslaApp/" + Version
)

// DebugFlag represents a set of debug bit flags that can be bitwise-ORed
// together to configure the different behaviors. This allows us to expand
// functionality in the future without introducing breaking changes.
type DebugFlag uint64

const (
	// DEBUG_FLAG_DISABLED disables all debug behaviors.
	DEBUG_FLAG_DISABLED DebugFlag = 0

	// DEBUG_FLAG_CAPTURE_LAST_REQUEST captures the last HTTP request made to the API
	// (if there was one) and makes it available via the LastAPIRequest()
	// method.
	//
	// This may increase memory usage / GC, as we'll be making a copy of the
	// full HTTP request body on each request and capturing it for inspection.
	DEBUG_FLAG_CAPTURE_LAST_REQUEST DebugFlag = 1 << 0

	// DEBUG_FLAG_CAPTURE_LAST_RESPONSE captures the last HTTP response from the API (if
	// there was one) and makes it available via the LastAPIResponse() method.
	//
	// This may increase memory usage / GC, as we'll be making a copy of the
	// full HTTP response body on each request and capturing it for inspection.
	DEBUG_FLAG_CAPTURE_LAST_RESPONSE DebugFlag = 1 << 1
)
